package GSM::SMS::Transport::XmlRpc;
use strict;
use vars qw( $VERSION );

use base qw( GSM::SMS::Transport::Transport );

$VERSION = '0.2';

=head1 NAME

GSM::SMS::Transport::XmlRpc - HTTP for Remote Serial modem 

=head1

This class implements a remote serial node. It needs to be used in conjunction with the remote node daemon (//TODO).

=cut
1;
